import unittest

from carball.analysis.analysis_manager import AnalysisManager

from carball.tests.utils import run_analysis_test_on_replay, get_specific_replays, get_specific_answers, assertNearlyEqual


class BoostTest(unittest.TestCase):

    def test_1_small_pad_collected(self):
        def test(analysis: AnalysisManager):
            proto_game = analysis.get_protobuf_data()
            player = proto_game.players[0]
            boost = player.stats.boost
            self.assertEqual(boost.num_small_boosts, 1)
            print(analysis)

        run_analysis_test_on_replay(test, get_specific_replays()["1_SMALL_PAD"])

    def test_1_large_pad_collected(self):
        def test(analysis: AnalysisManager):
            proto_game = analysis.get_protobuf_data()
            player = proto_game.players[0]
            boost = player.stats.boost
            self.assertEqual(boost.num_large_boosts, 1)
            print(analysis)

        run_analysis_test_on_replay(test, get_specific_replays()["1_LARGE_PAD"])

    def test_0_boost_collected(self):
        def test(analysis: AnalysisManager):
            proto_game = analysis.get_protobuf_data()
            player = proto_game.players[0]
            boost = player.stats.boost
            self.assertEqual(boost.num_small_boosts, 0)
            self.assertEqual(boost.num_large_boosts, 0)
            print(analysis)

        run_analysis_test_on_replay(test, get_specific_replays()["0_BOOST_COLLECTED"])

    def test_boost_used(self):
        def test(analysis: AnalysisManager, boost_value):
            proto_game = analysis.get_protobuf_data()
            player = proto_game.players[0]
            boost = player.stats.boost
            print("Predicted usage: {}, actual: {}".format(boost.boost_usage, boost_value))
            self.assertAlmostEqual(boost.boost_usage, boost_value, delta=1)
            # self.assertGreater(boost.average_boost_level, 0)

        run_analysis_test_on_replay(test, get_specific_replays()["BOOST_USED"] + get_specific_replays()["0_BOOST_USED"],
                                    answers=get_specific_answers()["BOOST_USED"] +
                                            get_specific_answers()["0_BOOST_USED"])

    def test_boost_feathered(self):
        def test(analysis: AnalysisManager, boost_value):
            proto_game = analysis.get_protobuf_data()
            player = proto_game.players[0]
            boost = player.stats.boost
            print("Predicted usage: {}, actual: {}".format(boost.boost_usage, boost_value))
            assertNearlyEqual(self, boost.boost_usage, boost_value, percent=3)
            # self.assertGreater(boost.average_boost_level, 0)

        run_analysis_test_on_replay(test, get_specific_replays()["BOOST_FEATHERED"],
                                    answers=get_specific_answers()["BOOST_FEATHERED"])

    def test_boost_wasted_collection(self):
        def test(analysis: AnalysisManager, boost_value):
            proto_game = analysis.get_protobuf_data()
            player = proto_game.players[0]
            boost = player.stats.boost
            self.assertAlmostEqual(boost.wasted_collection, boost_value, delta=1)
            # self.assertGreater(boost.average_boost_level, 0)
            print(analysis)

        # run_analysis_test_on_replay(test, get_specific_replays()["BOOST_WASTED_COLLECTION"],
        #                             answers=get_specific_answers()["BOOST_WASTED_COLLECTION"])

    def test_0_used(self):
        def test(analysis: AnalysisManager, boost_value):
            proto_game = analysis.get_protobuf_data()
            player = proto_game.players[0]
            boost = player.stats.boost
            self.assertEqual(boost.boost_usage, boost_value)
            print(analysis)

        run_analysis_test_on_replay(test, get_specific_replays()["0_BOOST_USED"],
                                    answers=get_specific_answers()["0_BOOST_USED"])


if __name__ == '__main__':
    unittest.main()



from typing import Dict

import pandas as pd

from carball.generated.api.stats.events_pb2 import Hit
from ....analysis.constants.field_constants import FieldConstants
from ....analysis.stats.stats import HitStat
from ....generated.api import game_pb2
from ....generated.api.player_pb2 import Player
from ....json_parser.game import Game


class TurnoverStat(HitStat):
    field_constants = FieldConstants()

    def initialize_hit_stat(self, game: Game, player_map: Dict[str, Player], data_frame: pd.DataFrame):
        pass

    def calculate_next_hit_stat(self, game: Game, proto_game: game_pb2.Game, saltie_hit: Hit, next_saltie_hit: Hit,
                                player_map: Dict[str, Player], hit_index: int):
        hits = proto_game.game_stats.hits
        hit_player = player_map[saltie_hit.player_id.id]
        second_hit_player = player_map[next_saltie_hit.player_id.id]

        # If there is a goal between 2nd hit and 3rd hit abort check
        if not next_saltie_hit.HasField("next_hit_frame_number") or hit_index + 2 >= len(hits):
            return

        third_hit_player = player_map[hits[hit_index + 2].player_id.id]
        if hit_player.is_orange != second_hit_player.is_orange and hit_player.is_orange != third_hit_player.is_orange:
            # this is a turnover!
            # if the hit occurred on the on the same half as my team
            my_half = (saltie_hit.ball_data.pos_y > 0) == hit_player.is_orange
            neutral_zone = self.field_constants.get_neutral_zone(saltie_hit.ball_data)
            self.assign_turnover(hit_player.stats.possession, my_half, neutral_zone)
            self.assign_turnover(proto_game.teams[hit_player.is_orange].stats.possession,
                                 my_half, neutral_zone)
            second_hit_player.stats.possession.won_turnovers += 1
            proto_game.teams[second_hit_player.is_orange].stats.possession.won_turnovers += 1

    def assign_turnover(self, possession_proto, is_turnover_my_half, is_neutral):
        possession_proto.turnovers += 1
        if is_turnover_my_half and not is_neutral:
            possession_proto.turnovers_on_my_half += 1
        elif not is_neutral:
            possession_proto.turnovers_on_their_half += 1
